@@ -24,7 +24,7 @@ from platformio import fs
 from platformio.compat import IS_WINDOWS, hashlib_encode_data
 
 
-class ConfigOption(object):  # pylint: disable=too-many-instance-attributes
+class ConfigOption:  # pylint: disable=too-many-instance-attributes
     def __init__(
         self,
         scope,
@@ -519,6 +519,13 @@ ProjectOptions = OrderedDict(
                 oldnames=["monitor_baud"],
                 default=9600,
             ),
+            ConfigEnvOption(
+                group="monitor",
+                name="monitor_parity",
+                description="A monitor parity checking",
+                type=click.Choice(["N", "E", "O", "S", "M"]),
+                default="N",
+            ),
             ConfigEnvOption(
                 group="monitor",
                 name="monitor_filters",
@@ -541,12 +548,30 @@ ProjectOptions = OrderedDict(
             ),
             ConfigEnvOption(
                 group="monitor",
-                name="monitor_flags",
-                description=(
-                    "The extra flags and options for `platformio device monitor` "
-                    "command"
-                ),
-                multiple=True,
+                name="monitor_eol",
+                description="A monitor end of line mode",
+                type=click.Choice(["CR", "LF", "CRLF"]),
+                default="CRLF",
+            ),
+            ConfigEnvOption(
+                group="monitor",
+                name="monitor_raw",
+                description="Disable encodings/transformations of device output",
+                type=click.BOOL,
+                default=False,
+            ),
+            ConfigEnvOption(
+                group="monitor",
+                name="monitor_echo",
+                description="Enable a monitor local echo",
+                type=click.BOOL,
+                default=False,
+            ),
+            ConfigEnvOption(
+                group="monitor",
+                name="monitor_encoding",
+                description="Custom encoding (e.g. hexlify, Latin1, UTF-8)",
+                default="UTF-8",
             ),
             # Library
             ConfigEnvOption(
